from colorama import Fore, Style
import sys
import logging
from sylo.models import Durations
from sylo.definitions import RANDOM_FONTS
from pyfiglet import figlet_format, FigletFont
from random import sample
logger = logging.getLogger(__name__)

THEME = Fore.BLUE


def set_theme(color: str = 'blue'):
    global THEME
    colors = {
        'red': Fore.RED,
        'lred': Fore.LIGHTRED_EX,
        'blue': Fore.BLUE,
        'lblue': Fore.LIGHTBLUE_EX,
        'green': Fore.GREEN,
        'lgreen': Fore.LIGHTGREEN_EX,
        'yellow': Fore.YELLOW,
        'magenta': Fore.MAGENTA,
        'white': Fore.WHITE,
        'black': Fore.BLACK,
    }
    logger.info('The theme is' + THEME)
    THEME = colors[color]


def print_message(message: str, timer_val: int = None):
    logger.debug(f'Printing message: {message} with a timer_val of {timer_val}')
    if message == "work_start":
        print(f"""{THEME}Ctrl-C to stop timer early{Style.RESET_ALL}
{Fore.RED}WORK{Style.RESET_ALL} for {Fore.YELLOW}{timer_val}{Style.RESET_ALL} minutes""")
    elif message == "rest_start":
        print(f"""{THEME}Ctrl-C to stop timer early{Style.RESET_ALL}
{Fore.GREEN}REST{Style.RESET_ALL} for {Fore.YELLOW}{timer_val}{Style.RESET_ALL} minutes""")
    elif message == "summary_and_quit":
        sys.stdout.write("\033[K")
        print(
            f"{THEME}Press ENTER to stop timer.{Style.RESET_ALL}",
        )
    elif message == "show_insights":
        print(f"\n{THEME}Press ENTER to return to main menu{Style.RESET_ALL}")
    elif message == "bar_header":
        print(f"{Fore.YELLOW}Weekly minutes{Style.RESET_ALL}")
    elif message == "heat_header":
        print(f"{Fore.YELLOW}Work heatmap{Style.RESET_ALL}")
    elif message == "cursor":
        return ">> "


def options():
    return """Additional commands;
S       --    Swap upcoming timer
I       --    Show the insights tab
Q       --    Quit SYLO
"""


def print_update(durations: Durations, mode: str, show_options: bool = False, show_graphs: bool = False):
    logger.debug(f'Printing update: {mode} with show_options {show_options}')
    if mode == 'rest':
        upcoming_timer_color = durations.rest.bar_color
    else:
        upcoming_timer_color = durations.work.bar_color

    if show_options:
        print_ops = options()
    else:
        print_ops = '.. or chose an optional command (H for help)'

    print(
        f"""Total {Fore.RED}work{Style.RESET_ALL} today:           {Fore.YELLOW}{int(durations.total_work_mins)} minutes{Style.RESET_ALL}
Total {Fore.GREEN}rest{Style.RESET_ALL} today:           {Fore.YELLOW}{int(durations.total_rest_mins)} minutes{Style.RESET_ALL}

Upcoming timer:     {upcoming_timer_color}{mode.upper()}{Style.RESET_ALL}

{THEME}Press {Style.RESET_ALL}{Fore.YELLOW}ENTER {Style.RESET_ALL}{THEME}to start the next timer{Style.RESET_ALL}

{THEME}{print_ops}{Style.RESET_ALL}

    """)


def ascii_header(font: str):
    return figlet_format('Sort Your Life Out', font=font, width=40)


def print_header_small(double: bool):
    if RANDOM_FONTS is True:
        font = random_ascii_header()
    else:
        font = ascii_header_small(font='alligator')
    if double is True:
        double_message = '>>>>>>>>>>>> DOUBLE SPEED MODE >>>>>>>>>>>>'
    else:
        double_message = ''
    print(f"{double_message}")
    print(f"""{THEME}{font}{Style.RESET_ALL}""")


def ascii_header_small(font: str):
    return figlet_format('SYLO', font=font, width=60)


def random_ascii_header():
    fonts = FigletFont.getFonts()
    random_fonts = sample(fonts, int(len(fonts)/2))
    for f in random_fonts:
        try:
            logger.debug(f'Output of shuffled fonts: {f}')
            return figlet_format('SYLO', font=f, width=40)
        except Exception as e:
            logger.debug(f'Output of shuffled fonts: {f} {e}')
            pass



import os
import datetime
from pathlib import Path

HOME_DIR = home = str(Path.home())
ROOT_DIR = os.path.dirname(os.path.abspath(__file__))

# TODAY = (datetime.datetime.now() + datetime.timedelta(days=1)).date()
TODAY = datetime.datetime.now().date()
TODAY_STR = TODAY.strftime("%Y-%m-%d")
ONE_WEEK_AGO = (datetime.datetime.now() - datetime.timedelta(days=7)).date()
ONE_YEAR_AGO = (datetime.datetime.now() - datetime.timedelta(days=364)).date()

DATA_FILE_LOCATION = f'{HOME_DIR}/.sylo/sessions.dat'
HOME_DIR_CREATE = f'mkdir {HOME_DIR}/.sylo'
DATA_FILE_CREATE = f'echo "{TODAY_STR},0,0" > {HOME_DIR}/.sylo/sessions.dat'


WELCOME_CHOICES = (
    "",
    "q",
    "s",
    "i"
)

TIMER_DEFAULTS = {
    "work": {
        "mins": 25,
        "secs": 1500,
    },
    "rest": {
        "mins": 5,
        "secs": 300,
    }
}

COUNTDOWN_INCREMENTS = 1


# Experimental Features

RANDOM_FONTS = False

METRICS_HEAT = 'termgraph --calendar --start-dt "%s" %s '

METRICS_BAR = 'termgraph %s --color {red,green} --start-dt "%s"'



from setuptools import setup


VERSION = "1.5.0_beta1"


with open("README.md") as f:
    readme = f.read()


setup(
    name="sylo",
    version=VERSION,
    description="SYLO Python Pomodoro Timer",
    long_description=readme,
    long_description_content_type="text/markdown",  # This is important!
    url="http://github.com/rob-parker-what/sylo",
    author="rob-parker-what",
    author_email="robparkerwhat.dev@gmail.com",
    license="MIT",
    packages=["sylo"],
    entry_points={
        "console_scripts": ["sylo=sylo.cmdline:main"],
    },
    install_requires=[
        "colorama==0.4.4",
        "simpleaudio==1.0.4",
        "beepy==1.0.7",
        "tqdm==4.61.1",
        "pytimedinput==1.3.1",
        "pyfiglet==0.8.post1",
        "pandas==1.1.5",
        "termgraph==0.5.1",
    ],
    package_data={"sylo": ["data"]},
    zip_safe=False,
    python_requires=">=3.7",
    keywords="pomodoro tomato timer terminal pomodoro-timer",
    classifiers=[
        "Intended Audience :: Developers",
        "License :: OSI Approved :: MIT License",
        "Programming Language :: Python",
        "Topic :: Software Development",
        "Operating System :: POSIX",
        "Operating System :: Unix",
        "Operating System :: MacOS",
    ],
)



import os
import logging
import subprocess
import sys

logger = logging.getLogger(__name__)


def get_input(text):
    return input(text)


def run_subprocess(script: str):
    if "rm " in script:
        logger.warning('UNSAFE SCRIPT')
        sys.exit()
    try:
        subprocess.run(script,
                       shell=True,
                       check=True,
                       timeout=30
                       )
    except subprocess.CalledProcessError as e:
        print(e.output)
        logger.warning(e.output)


def clear_screen():
    os.system("cls" if os.name == "nt" else "clear")


def mins_to_secs(minutes: int):
    return minutes * 60


def flip_mode(mode: str):
    if mode == 'work':
        logger.debug(f'flip_mode took {mode} returned rest')
        return 'rest'
    else:
        logger.debug(f'flip_mode took {mode} returned work')
        return 'work'


def check_for_file(path):
    is_file = os.path.isfile(path)
    logger.info(f'check_for_file {is_file}')
    return is_file



import logging
from dataclasses import dataclass
from colorama import Fore
from sylo.definitions import TIMER_DEFAULTS, DATA_FILE_LOCATION, TODAY_STR
import pandas as pd

logger = logging.getLogger(__name__)


@dataclass
class Rest:
    mins: int = TIMER_DEFAULTS["rest"]["mins"]
    secs: int = TIMER_DEFAULTS["rest"]["secs"]
    bar_color: str = Fore.GREEN


@dataclass
class Work:
    mins: int = TIMER_DEFAULTS["work"]["mins"]
    secs: int = TIMER_DEFAULTS["work"]["secs"]
    bar_color: str = Fore.RED


@dataclass
class Durations:
    work = Work
    rest = Rest
    total_work_mins: float = 0
    total_rest_mins: float = 0

    def __post_init__(self):
        self.work = Work()
        self.rest = Rest()


class DataPersistence:

    def __init__(self, data_file: str = DATA_FILE_LOCATION):
        self.data_file = data_file
        self.file_cols = ["date", "time_worked", "time_rested"]

    def data_refresh(self, durations: Durations):
        new_df = self.fetch_and_update_values(durations, TODAY_STR)
        new_df.to_csv(DATA_FILE_LOCATION, sep=',', mode='w', header=False)

    def data_load_on_boot(self, durations: Durations):
        df = self._read_file()
        try:
            filtered_dates = df.loc[TODAY_STR]
            logger.info('data_load_on_boot Found data for today')
            work_time_today = float(filtered_dates['time_worked'])
            rest_time_today = float(filtered_dates['time_rested'])
            logger.info(f'Work time loaded on boot: {work_time_today}')
            logger.info(f'Rest time loaded on boot: {rest_time_today}')
            durations.total_work_mins = work_time_today
            durations.total_rest_mins = rest_time_today
        except Exception:
            logger.info('data_load_on_boot NO DATA for today')
            filtered_dates = None
        if filtered_dates is None:
            durations.total_rest_mins = 0
            durations.total_work_mins = 0

    def _read_file(self):
        df = pd.read_csv(self.data_file, header=None, sep=',')
        df.columns = self.file_cols
        df.set_index('date', inplace=True, drop=True)
        return df

    def fetch_and_update_values(self, durations: Durations, date: str = TODAY_STR):
        df = self._read_file()
        new_df = pd.DataFrame({
            'date': [date],
            'time_worked': [durations.total_work_mins],
            'time_rested': [durations.total_rest_mins]
        }, columns=self.file_cols, dtype='float')
        new_df.set_index('date', inplace=True, drop=True)
        out = new_df.combine_first(df)
        return out
